@@ -1,6 +1,7 @@
 # UPDATED DS
 #  v0.2
-#pylint: disable=no-member
+# pylint: disable=no-member
+# pylint: disable=eval-used
 # singleton: string
 # proposition: list of strings
 
@@ -21,11 +22,11 @@ import copy
 from itertools import chain, combinations
 
 
-
 class BOE:
     """
     A class to represent a DS-Theoretic Body of Evidence
     """
+
     def __init__(self, singletons):
         """
         Constructor
@@ -42,7 +43,6 @@ class BOE:
         # Lookup table containing keys of singletons in the masses_dsvector
         self._power = self._initialize_power()
 
-
     @staticmethod
     def _default_mass():
         """
@@ -50,7 +50,7 @@ class BOE:
         """
         return 0
 
-    #-------------------------------------
+    # -------------------------------------
     # DS-Theoretic Properties
 
     @property
@@ -98,7 +98,7 @@ class BOE:
         """
         return sum(self.dsvector.values())
 
-    #----------------------------------
+    # ----------------------------------
     # Key DS-Theoretic Operations
 
     def set_mass(self, proposition, mass):
@@ -112,6 +112,13 @@ class BOE:
         index = self._get_index_from_dsvector(proposition)
         self.dsvector = (index, mass)
 
+    def set_masses(self, dsvector_as_dict):
+        """
+        Sets masses for several propositions
+        from a json entry
+        """
+        for key, value in dsvector_as_dict.items():
+            self.set_mass(eval(key), value)
 
     def set_mass_theta(self, mass):
         """
@@ -120,7 +127,6 @@ class BOE:
         index = self._get_index_from_dsvector(self.frame)
         self.dsvector = (index, mass)
 
-
     def get_mass(self, proposition):
         """
         Get UNNORMALIZED mass for a proposition or a singleton
@@ -129,13 +135,12 @@ class BOE:
         index = self._get_index_from_dsvector(proposition)
         return self.dsvector[index]
 
-
     def get_normalized_mass(self, proposition):
         """
         Get normalized mass for a proposition
         """
         proposition = [x.lower() for x in proposition]
-        return self.get_mass(proposition)/self.normalizing_constant
+        return self.get_mass(proposition) / self.normalizing_constant
 
     def get_masses(self):
         """
@@ -155,7 +160,7 @@ class BOE:
         masses = self.get_masses()
         normalized_masses = {}
         for key, value in masses.items():
-            nmass = value/self.normalizing_constant
+            nmass = value / self.normalizing_constant
             normalized_masses.update({key: nmass})
         return normalized_masses
 
@@ -165,7 +170,10 @@ class BOE:
         """
         normalized_dsvector = {}
         for key, value in self.dsvector.items():
-            normalized_mass = value/self.normalizing_constant
+            if self.normalizing_constant != 0:
+                normalized_mass = value / self.normalizing_constant
+            else:
+                normalized_mass = value
             normalized_dsvector.update({key: normalized_mass})
         return normalized_dsvector
 
@@ -181,7 +189,6 @@ class BOE:
             core.append(self.get_prop_from_dsvector(number))
         return core
 
-
     def belief(self, proposition):
         """
         Returns belief of a proposition
@@ -195,8 +202,7 @@ class BOE:
         for subset in non_zero_subsets:
             mass = self.get_mass(subset)
             belief += mass
-        return belief/self.normalizing_constant
-
+        return belief / self.normalizing_constant
 
     def plausibility(self, proposition):
         """
@@ -210,7 +216,7 @@ class BOE:
         for overlapping in non_zero_overlaps:
             mass = self.get_mass(overlapping)
             plausibility += mass
-        return plausibility/self.normalizing_constant
+        return plausibility / self.normalizing_constant
 
     def uncertainty(self, proposition):
         """
@@ -231,7 +237,6 @@ class BOE:
             uncertainties.update({str(proposition): uncertainty})
         return uncertainties
 
-
     def conditional_mass(self, proposition_b, proposition_a):
         """
         Returns conditional mass (b given a)
@@ -250,9 +255,9 @@ class BOE:
             if temp:
                 pl_a_minus_b = self.plausibility(temp)
 
-        mass_b_given_a = mass_b/(mass_b + pl_a_minus_b)
-        return mass_b_given_a
-
+            mass_b_given_a = mass_b / (mass_b + pl_a_minus_b)
+            return mass_b_given_a
+        return 0.0
 
     def update(self, new_frame, alpha):
         """
@@ -287,7 +292,6 @@ class BOE:
             term2 = (1 - alpha) * total
             self.set_mass(prop_b, term1 + term2)
 
-
     def update_stream(self, list_boes, alpha):
         """
         CUE update for a list of boes
@@ -297,10 +301,7 @@ class BOE:
             self.update(boe, alpha)
             print(f"Unc at [{idx}]: {self.get_uncertainties()}")
 
-
-
-
-    #------------- SPECIALIZED DS HELPERS -------------------------
+    # ------------- SPECIALIZED DS HELPERS -------------------------
 
     def _initialize_power(self):
         """
@@ -311,11 +312,10 @@ class BOE:
         """
         power = []
         for i in range(len(self.frame)):
-            j = 2 ** i
+            j = 2**i
             power.append(j)
         return power
 
-
     def _get_index_from_dsvector(self, proposition):
         """
         Returns a DSVector key of a proposition.
@@ -329,11 +329,12 @@ class BOE:
                 idx = self.frame.index(singleton)
                 key += self.power[idx]
             except:
-                raise ValueError('One of the singletons in the' +\
-                                 'proposition is not found in the frame')
+                raise ValueError(
+                    "One of the singletons in the"
+                    + "proposition is not found in the frame"
+                )
         return key
 
-
     def get_prop_from_dsvector(self, index):
         """
         Returns a proposition given a DSVector index
@@ -342,7 +343,6 @@ class BOE:
         singletons = [self.frame[i] for i in indexes]
         return singletons
 
-
     def _get_subsets_from_dsvector(self, proposition):
         """
         Returns all the subsets of a proposition that have
@@ -352,7 +352,7 @@ class BOE:
         with non-zero mass in the dsvector
         """
         subsets = []
-        for key, _value  in self.dsvector.items():
+        for key, _value in self.dsvector.items():
             key_proposition = self.get_prop_from_dsvector(key)
             if set(key_proposition).issubset(set(proposition)):
                 subsets.append(key_proposition)
@@ -369,8 +369,7 @@ class BOE:
                 overlapping.append(key_proposition)
         return overlapping
 
-
-    #-------------------- COMBINATION -------------------------
+    # -------------------- COMBINATION -------------------------
 
     def conjunctive_form(self, another_boe, proposition):
         """
@@ -389,7 +388,6 @@ class BOE:
                     mass += mass1 * mass2
         return mass
 
-
     def disjunctive_form(self, another_boe, proposition):
         """
         m(self union other_boe)
@@ -407,7 +405,6 @@ class BOE:
                     mass += mass1 * mass2
         return mass
 
-
     def conflict(self, another_boe):
         """
         K (or conflict) for Combination rules
@@ -436,7 +433,9 @@ class BOE:
         if self.conflict(another_boe) == 1:
             return None
 
-        return self.conjunctive_form(another_boe, proposition)/(1 - self.conflict(another_boe))
+        return self.conjunctive_form(another_boe, proposition) / (
+            1 - self.conflict(another_boe)
+        )
 
     def yager(self, another_boe, proposition):
         """
@@ -450,7 +449,9 @@ class BOE:
             return 0
 
         if proposition == another_boe.frame:
-            return self.conjunctive_form(another_boe, proposition) + self.conflict(another_boe)
+            return self.conjunctive_form(another_boe, proposition) + self.conflict(
+                another_boe
+            )
 
         return self.conjunctive_form(another_boe, proposition)
 
@@ -511,35 +512,37 @@ class BOE:
 
         return term1 + term2
 
-
     def gamma(self, another_boe, proposition1, proposition2):
         """
         Gamma(B,C) useful for PCR
         """
 
-        term1_numerator = (self.get_normalized_mass(proposition1)**2) * \
-        another_boe.get_normalized_mass(proposition2)
+        term1_numerator = (
+            self.get_normalized_mass(proposition1) ** 2
+        ) * another_boe.get_normalized_mass(proposition2)
 
-        denominator1 = self.get_normalized_mass(proposition1) + \
-        another_boe.get_normalized_mass(proposition2)
+        denominator1 = self.get_normalized_mass(
+            proposition1
+        ) + another_boe.get_normalized_mass(proposition2)
 
-        term2_numerator = (another_boe.get_normalized_mass(proposition1)**2) * \
-        self.get_normalized_mass(proposition2)
+        term2_numerator = (
+            another_boe.get_normalized_mass(proposition1) ** 2
+        ) * self.get_normalized_mass(proposition2)
 
-        denominator2 = another_boe.get_normalized_mass(proposition1) + \
-        self.get_normalized_mass(proposition2)
+        denominator2 = another_boe.get_normalized_mass(
+            proposition1
+        ) + self.get_normalized_mass(proposition2)
 
         ratio1 = 0
         if denominator1 != 0:
-            ratio1 = term1_numerator/denominator1
+            ratio1 = term1_numerator / denominator1
 
         ratio2 = 0
         if denominator2 != 0:
-            ratio2 = term2_numerator/denominator2
+            ratio2 = term2_numerator / denominator2
 
         return ratio1 + ratio2
 
-
     def pcr5_multisource(self, list_boes):
         """
         Returns a fused BOE by repeatedly calling pcr5()
@@ -560,8 +563,59 @@ class BOE:
             boe1 = new_boe
         return boe1
 
+    def yager_multisource(self, list_boes):
+        """
+        Returns a fused BOE by repeatedly calling yager()
 
-    #------------- GENERIC HELPERS -------------------------
+        """
+        powers = list(self._powerset(self.frame))
+
+        boe1 = copy.copy(self)
+        # Doing YAGER in pairs
+        for _idx, boe2 in enumerate(list_boes):
+            new_boe = BOE(self.frame)
+            for proposition in powers:
+                new_boe.set_mass(proposition, boe1.yager(boe2, proposition))
+            boe1 = new_boe
+        return boe1
+
+    def dcr_multisource(self, list_boes):
+        """
+        Returns a fused BOE by repeatedly calling dcr()
+
+        """
+        powers = list(self._powerset(self.frame))
+
+        boe1 = copy.copy(self)
+        # Doing DCR in pairs
+        for _idx, boe2 in enumerate(list_boes):
+            new_boe = BOE(self.frame)
+            for proposition in powers:
+                new_boe.set_mass(proposition, boe1.dcr(boe2, proposition))
+            boe1 = new_boe
+        return boe1
+
+    def dubois_prade_multisource(self, list_boes):
+        """
+        Returns a fused BOE by repeatedly calling dubois_prade()
+
+        """
+        powers = list(self._powerset(self.frame))
+
+        boe1 = copy.copy(self)
+        # Doing Dubois in pairs
+        for _idx, boe2 in enumerate(list_boes):
+            new_boe = BOE(self.frame)
+            for proposition in powers:
+                new_boe.set_mass(proposition, boe1.dubois_prade(boe2, proposition))
+            boe1 = new_boe
+        return boe1
+
+
+
+
+
+    # ------------- GENERIC HELPERS -------------------------
 
     @staticmethod
     def _find_powers_of_2(number):
@@ -580,7 +634,6 @@ class BOE:
                 indexes.append(i)
         return indexes
 
-
     @staticmethod
     def _powerset(iterable):
         """
@@ -591,5 +644,6 @@ class BOE:
         """
 
         iterable_list = list(iterable)
-        return chain.from_iterable(combinations(iterable_list, r) \
-                                   for r in range(len(iterable_list)+1))
+        return chain.from_iterable(
+            combinations(iterable_list, r) for r in range(len(iterable_list) + 1)
+        )
